;;;
;;; comb_tst.lisp : combination のテスト
;;;
;;; Copyright (C) 2023 Makoto Hiroi
;;;
;;; Released under the MIT license
;;; https://opensource.org/license/mit/
;;;
(provide :comb_tst)
(defpackage :comb_tst (:use :cl :combination :mintst))
(in-package :comb_tst)
(export '(test))

(defvar xs1 '((a b c) (a c b) (b a c) (b c a) (c a b) (c b a)))
(defvar xs2 '((A A) (A B) (A C) (B A) (B B) (B C) (C A) (C B) (C C)))
(defvar xs3 '((B A D C) (B C D A) (B D A C) (C A D B) (C D A B)
	      (C D B A) (D A B C) (D C A B) (D C B A)))
(defvar xs4 '((A B C) (A B D) (A B E) (A C D) (A C E)
	      (A D E) (B C D) (B C E) (B D E) (C D E)))
(defvar xs5 '((A A) (A B) (A C) (A D) (B B) (B C) (B D) (C C) (C D) (D D)))
(defvar xs6 '((A D) (A E) (A F) (B D) (B E) (B F) (C D) (C E) (C F)))

(defun test ()
  (initial)
  (run (let ((a nil))
	 (permutation (lambda (x) (push x a)) 3 '(a b c))
	 (reverse a))
       xs1)
  (run (permutation-set 3 '(a b c)) xs1)
  (run (let ((a nil))
	 (repeated-permutation (lambda (x) (push x a)) 2 '(a b c))
	 (reverse a))
       xs2)
  (run (repeated-permutation-set 2 '(a b c)) xs2)
  (run (let ((a nil))
	 (derangement (lambda (x) (push x a)) '(a b c d))
	 (reverse a))
       xs3)
  (run (derangement-set '(a b c d)) xs3)
  (run (let ((a nil))
	 (combination (lambda (x) (push x a)) 3 '(a b c d e))
	 (reverse a))
       xs4)
  (run (combination-set 3 '(a b c d e)) xs4)
  (run (let ((a nil))
	 (repeated-combination (lambda (x) (push x a)) 2 '(a b c d))
	 (reverse a))
       xs5)
  (run (repeated-combination-set 2 '(a b c d)) xs5)
  (run (let ((a nil))
	 (product (lambda (x) (push x a)) '(a b c) '(d e f))
	 (reverse a))
       xs6)
  (run (product-set '(a b c) '(d e f)) xs6)

  (run (mapcar #'factorial '(0 10 11 20)) '(1 3628800 39916800 2432902008176640000))
  (run (mapcar #'fibonacci '(0 10 20 40 80)) '(0 55 6765 102334155 23416728348467685))
  (run (mapcar (lambda (xs) (apply #'permutation-number xs)) '((10 4) (10 5) (10 6) (20 10)))
       '(5040 30240 151200 670442572800))
  (run (mapcar (lambda (xs) (apply #'combination-number xs)) '((5 3) (10 5) (20 10) (30 15) (40 20)))
       '(10 252 184756 155117520 137846528820))
  (run (mapcar (lambda (xs) (apply #'repeated-combination-number xs)) '((5 3) (10 5) (20 10) (30 15)))
       '(35 2002 20030010 229911617056))
  (run (mapcar (lambda (x) (catalan-number x)) '(0 1 2 3 10 20 30))
       '(1 1 2 5 16796 6564120420 3814986502092304))
  (run (mapcar (lambda (x) (montmort-number x)) '(1 2 3 4 10 20))
       '(0 1 2 9 1334961 895014631192902121))
  (run (mapcar (lambda (xs) (apply #'stirling2-number xs))
	       '((0 0) (1 0) (1 1) (5 1) (5 3) (5 5) (20 10)))
       '(1 0 1 1 25 1 5917584964655))
  (run (mapcar (lambda (x) (bell-number x)) '(0 1 2 3 4 10 20))
       '(1 1 2 5 15 115975 51724158235372))
  (run (mapcar (lambda (x) (partition-number x)) '(1 2 3 4 10 100 200))
       '(1 2 3 5 42 190569292 3972999029388))
  (final))

