;;;
;;; deque_tst.lisp : deque のテスト
;;;
;;; Copyright (C) 2023 Makoto Hiroi
;;;
;;; Released under the MIT license
;;; https://opensource.org/license/mit/
;;;
(provide :deque_tst)
(defpackage :deque_tst (:use :cl :deque :mintst))
(in-package :deque_tst)
(export '(test))

(defvar d nil)
(defvar q nil)

(defun test ()
  (initial)
  (run (dlist-p (setq d (make-dlist))) t)
  (run (dlist-emptyp d) t)
  (run (dlist-count d) 0)
  (run (dlist-insert-at d 0 1) 1)
  (run (dlist-emptyp d) nil)
  (run (dlist-count d) 1)
  (run (dlist-nth d 0) 1)
  (run (dlist-nth d 0 :from-end t) 1)
  (run (dolist (x '(2 3 4) (dlist-count d)) (dlist-insert-at d 0 x :from-end t)) 4)
  (run (dolist (x '(5 6 7 8) (dlist-count d)) (dlist-insert-at d 0 x)) 8)
  (run (let ((a nil))
	 (dotimes (x (dlist-count d) a) (push (dlist-nth d x) a)))
       '(4 3 2 1 5 6 7 8))
  (run (let ((a nil))
	 (dotimes (x (dlist-count d) a) (push (dlist-nth d x :from-end t) a)))
       '(8 7 6 5 1 2 3 4))
  (run (let ((a nil))
	 (dlist-mapc d (lambda (x) (push x a)))
	 a)
       '(4 3 2 1 5 6 7 8))
  (run (let ((a nil))
	 (dlist-mapc d (lambda (x) (push x a)) :from-end t)
	 a)
       '(8 7 6 5 1 2 3 4))
  (run (dlist-delete-at d 0) 8)
  ;; 7 6 5 1 2 3 4
  (run (dlist-delete-at d 0 :from-end t) 4)
  ;; 7 6 5 1 2 3
  (run (dlist-delete-at d 2) 5)
  ;; 7 6 1 2 3
  (run (dlist-delete-at d 2 :from-end t) 1)
  (run (dlist->list d) '(7 6 2 3))
  (run (dlist-count d) 4)
  (run (dlist-fold d (lambda (a x) (cons x a)) nil) '(3 2 6 7))
  (run (dlist-fold d #'cons nil :from-end t) '(7 6 2 3))
  (run (dotimes (x (dlist-count d) (dlist->list d)) (dlist-set d x x))
       '(0 1 2 3))
  (run (dotimes (x (dlist-count d) (dlist->list d)) (dlist-set d x x :from-end t))
       '(3 2 1 0))
  (run (dlist-clear d) 0)
  (run (dlist-emptyp d) t)
  (run (dlist-count d) 0)

  ;; 探索
  (run (dlist-count (setq d (list->dlist '((1 2) (3 4) (5 6))))) 3)
  (run (dlist-exists d 3) nil)
  (run (dlist-exists d (list 3 4)) nil)
  (run (dlist-exists d (list 3 4) :test #'equal) t)
  (run (dlist-find d 4 :key #'car) nil)
  (run (dlist-find d 4 :key #'second) '(3 4))
  (run (dlist-find-if d #'evenp :key #'car) nil)
  (run (dlist-find-if d #'evenp :key #'second) '(1 2))
  (run (dlist-find-if d #'evenp :key #'second :from-end t) '(5 6))
  (run (dlist-find-if-not d #'evenp :key #'car) '(1 2))
  (run (dlist-find-if-not d #'evenp :key #'second) nil)
  (run (dlist-find-if-not d #'evenp :key #'car :from-end t) '(5 6))

  ;; 削除
  (run (dlist-count (setq d (list->dlist '(1 2 3 1 2 3 1 2 3)))) 9)
  (run (dlist->list (dlist-delete d 1)) '(2 3 2 3 2 3))
  (run (dlist-count d) 6)
  (run (dlist->list (dlist-delete d 2 :count 1)) '(3 2 3 2 3))
  (run (dlist-count d) 5)
  (run (dlist->list (dlist-delete d 2 :count 1 :from-end t)) '(3 2 3 3))
  (run (dlist-count d) 4)
  (run (dlist-count (setq d (list->dlist '(1 2 3 4 5 6)))) 6)
  (run (dlist->list (dlist-delete-if d #'evenp)) '(1 3 5))
  (run (dlist-count d) 3)
  (run (dlist-count (setq d (list->dlist '(1 2 3 4 5 6)))) 6)
  (run (dlist->list (dlist-delete-if-not d #'evenp)) '(2 4 6))
  (run (dlist-count d) 3)
  (run (dolist (x '(2 4 6) (dlist-emptyp d)) (dlist-delete d x)) t)

  ;; 両端キュー
  (run (deque-p (setq q (make-deque))) t)
  (run (deque-emptyp q) t)
  (run (deque-count q) 0)
  (run (push-front q 1) 1)
  (run (deque-emptyp q) nil)
  (run (deque-count q) 1)
  (run (push-back q 10) 10)
  (run (deque-emptyp q) nil)
  (run (deque-count q) 2)
  (run (peek-front q) 1)
  (run (peek-back q) 10)
  (run (dolist (x '(2 3 4) (deque-count q)) (push-front q x)) 5)
  (run (dolist (x '(20 30 40) (deque-count q)) (push-back q x)) 8)
  (run (peek-front q) 4)
  (run (peek-back q) 40)
  (run (let ((a nil)) (dotimes (x 4 a) (push (pop-front q) a)))
       '(1 2 3 4))
  (run (let ((a nil)) (dotimes (x 4 a) (push (pop-back q) a)))
       '(10 20 30 40))
  (run (deque-emptyp q) t)
  (run (deque-count q) 0)
  (run (push-front q 10) 10)
  (run (deque-clear q) 0)
  (run (deque-count q) 0)
  (run (deque-emptyp q) t)
  (final))
	
