;;;
;;; heap_tst.lisp : heap のテスト
;;;
;;; Copyright (C) 2023 Makoto Hiroi
;;;
;;; Released under the MIT license
;;; https://opensource.org/license/mit/
;;;
(provide :heap_tst)
(defpackage :heap_tst (:use :cl :mintst :heap))
(in-package :heap_tst)
(export '(test))

(defvar h nil)
(defvar h1 nil)
(defvar h2 nil)
(defvar h3 nil)

(defun test-sub1 ()
  (run (heap-emptyp h) t)
  (run (heap-count h) 0)
  (run (dolist (x '(5 6 4 7 3 8 2 9 1 0) (heap-count h)) (heap-push h x)) 10)
  (run (heap-emptyp h) nil)
  (run (heap-peek h) 0)
  (run (do ((a nil))
	   ((heap-emptyp h) (reverse a))
	   (push (heap-pop h) a))
       '(0 1 2 3 4 5 6 7 8 9))
  (run (heap-emptyp h) t)
  (run (heap-count h) 0))

(defun test-sub2 ()
  (run (heap-emptyp h) t)
  (run (heap-count h) 0)
  (run (dolist (x '((5 a) (6 b) (4 c) (7 d) (0 e)
                    (3 f) (8 g) (2 h) (9 i) (1 j))
		  (heap-count h))
	 (heap-push h x))
       10)
  (run (heap-emptyp h) nil)
  (run (heap-peek h) '(9 i))
  (run (do ((a nil))
	   ((heap-emptyp h) (reverse a))
	   (push (heap-pop h) a))
       '((9 I) (8 G) (7 D) (6 B) (5 A)
	 (4 C) (3 F) (2 H) (1 J) (0 E)))
  (run (heap-emptyp h) t)
  (run (heap-count h) 0))

(defun test-sub3 ()
  (run (dolist (x '(2 4 0 8 6) (heap-count h1)) (heap-push h1 x)) 5)
  (run (dolist (x '(3 1 5 9 7) (heap-count h2)) (heap-push h2 x)) 5)
  (run (heap-count (setq h3 (heap-merge h1 h2))) 10)
  (run (do ((a nil))
	   ((heap-emptyp h3) (reverse a))
	   (push (heap-pop h3) a))
       '(0 1 2 3 4 5 6 7 8 9)))

(defun test ()
  (initial)
  (run (heap-p (setq h (make-heap))) t)
  (test-sub1)
  (run (heap-p (setq h (make-heap :key #'car :elt> #'<))) t)
  (test-sub2)
  (run (heap-p (setq h1 (make-heap))) t)
  (run (heap-p (setq h2 (make-heap))) t)
  (test-sub3)
  ;;
  (run (leftist-heap-p (setq h (make-leftist-heap))) t)
  (test-sub1)
  (run (leftist-heap-p (setq h (make-leftist-heap :key #'car :elt< #'>))) t)
  (test-sub2)
  (run (leftist-heap-p (setq h1 (make-leftist-heap))) t)
  (run (leftist-heap-p (setq h2 (make-leftist-heap))) t)
  (test-sub3)
  ;;
  (run (skew-heap-p (setq h (make-skew-heap))) t)
  (test-sub1)
  (run (skew-heap-p (setq h (make-skew-heap :key #'car :elt< #'>))) t)
  (test-sub2)
  (run (skew-heap-p (setq h1 (make-skew-heap))) t)
  (run (skew-heap-p (setq h2 (make-skew-heap))) t)
  (test-sub3)
  (final))

