;;;
;;; lazy_tst.lisp : lazy のテスト
;;;
;;; Copyright (C) 2023 Makoto Hiroi
;;;
;;; Released under the MIT license
;;; https://opensource.org/license/mit/
;;;
(provide :lazy_tst)
(defpackage :lazy_tst (:use :cl :lazy :mintst))
(in-package :lazy_tst)
(export '(test))

(defun square (x) (* x x))

(defun test ()
  (initial)
  (run (tolist (lcons 1 nil)) '(1))
  (run (lcar (lcons 1 nil)) 1)
  (run (lcdr (lcons 1 nil)) nil)
  (run (tolist (llist 1 2 3 4)) '(1 2 3 4))
  (run (tolist (fromlist '(1 2 3 4))) '(1 2 3 4))
  (run (liota 0) nil)
  (run (tolist (liota 3)) '(0 1 2))
  (run (tolist (liota 4 :start 1)) '(1 2 3 4))
  (run (tolist (liota 5 :start 3 :step 2)) '(3 5 7 9 11))
  (run (ltabulate 0 #'identity) nil)
  (run (tolist (ltabulate 3 #'identity)) '(0 1 2))
  (run (tolist (ltabulate 4 #'identity :start 1)) '(1 2 3 4))
  (run (tolist (ltabulate 5 #'square :start 2 :step 2)) '(4 16 36 64 100))
  (run (tolist (lunfold (lambda (x) (> x 4)) #'identity #'1+ 0))
       '(0 1 2 3 4))
  (run (tolist (lunfold (lambda (x) (> x 5)) #'square #'1+ 1))
       '(1 4 9 16 25))
  (run (tolist (lunfold (lambda (xs) (> (car xs) 3000000))
			#'car
			(lambda (xs) (list (second xs) (apply #'+ xs)))
			'(0 1)))
       '(0 1 1 2 3 5 8 13 21 34 55 89 144 233 377 610 987 1597 2584 4181 6765 10946
	 17711 28657 46368 75025 121393 196418 317811 514229 832040 1346269 2178309))
  (run (lnth 0 (llist 10 20 30 40 50)) 10)
  (run (lnth 2 (llist 10 20 30 40 50)) 30)
  (run (lnth 4 (llist 10 20 30 40 50)) 50)
  (run (llength nil) 0)
  (run (llength (liota 5)) 5)
  (run (llength (liota 10)) 10)
  (run (tolist (lappend (llist 1 2 3) (llist 4 5 6))) '(1 2 3 4 5 6))
  (run (tolist (lappend-delay (llist 1 2 3) (delay (llist 4 5 6)))) '(1 2 3 4 5 6))
  (run (tolist (lzip (llist 1 2 3) (llist 4 5 6))) '((1 4) (2 5) (3 6)))
  (run (tolist (ltake (liota nil) 5)) '(0 1 2 3 4))
  (run (tolist (ltake (ldrop (liota nil) 5) 5)) '(5 6 7 8 9))
  ;; 高階関数
  (run (tolist (ltake (lmap #'square (liota nil)) 5)) '(0 1 4 9 16))
  (run (tolist (ltake (lflatmap (lambda (x) (list x x)) (liota nil)) 6)) '(0 0 1 1 2 2))
  (run (tolist (ltake (lfilter #'evenp (liota nil)) 4)) '(0 2 4 6))
  (run (lfold-left #'+ 0 (llist 1 2 3 4 5)) 15)
  (run (lfold-left #'cons nil (llist 1 2 3 4 5))
       '(((((NIL . 1) . 2) . 3) . 4) . 5))
  (run (lfold-right #'+ 0 (llist 1 2 3 4 5)) 15)
  (run (lfold-right #'cons nil (llist 1 2 3 4 5))
       '(1 2 3 4 5))
  (run (tolist (ltake (lscan-left #'* 1 (liota nil :start 1)) 5)) '(1 1 2 6 24))
  (run (let ((a nil))
	 (lmapc (lambda (x) (push x a)) (llist 1 2 3 4 5))
	 (reverse a))
       '(1 2 3 4 5))
  (run (tolist (ltake-while (lambda (x) (< x 5)) (liota nil))) '(0 1 2 3 4))
  (run (tolist (ltake (ldrop-while (lambda (x) (< x 5)) (liota nil)) 5)) '(5 6 7 8 9))
  (run (levery #'evenp (llist 2 4 6 8 10)) t)
  (run (levery #'evenp (llist 2 4 5 8 10)) nil)
  (run (lsome #'oddp (llist 2 4 5 8 10)) t)
  (run (lsome #'oddp (llist 2 4 6 8 10)) nil)
  ;; 集合演算
  (run (tolist (lunion (llist 1 2 3 4) (llist 3 4 5 6))) '(1 2 3 4 5 6))
  (run (tolist (lintersect (llist 1 2 3 4) (llist 3 4 5 6))) '(3 4))
  (run (tolist (ldifferent (llist 1 2 3 4) (llist 3 4 5 6))) '(1 2))
  (run (tolist (ldifferent (llist 3 4 5 6) (llist 1 2 3 4))) '(5 6))
  (run (tolist (ldifferent (llist 1 2 3 4) (llist 1 2 3 4))) nil)
  ;; 順列と組み合わせ
  (run (tolist (lpermutation 4 (llist 'a 'b 'c 'd)))
       '((A B C D) (A B D C) (A C B D) (A C D B) (A D B C) (A D C B) (B A C D)
	 (B A D C) (B C A D) (B C D A) (B D A C) (B D C A) (C A B D) (C A D B)
	 (C B A D) (C B D A) (C D A B) (C D B A) (D A B C) (D A C B) (D B A C)
	 (D B C A) (D C A B) (D C B A)))
  (run (tolist (lcombination 3 (llist 'a 'b 'c 'd 'e)))
       '((A B C) (A B D) (A B E) (A C D) (A C E) (A D E) (B C D) (B C E) (B D E) (C D E)))
  (final))

