;;;
;;; mintst.lisp : 簡単なテストツール
;;;
;;; Copyright (c) 2023 Makoto Hiroi
;;;
;;; Released under the MIT license
;;; https://opensource.org/license/mit/
;;;
(provide :mintst)
(defpackage :mintst (:use :cl))
(in-package :mintst)
(export '(initial
	  run
	  final))

;;; 結果
(defvar *test-count* 0)
(defvar *ok-count* 0)
(defvar *ng-count* 0)
(defvar *err-count* 0)

;;; 初期化
(defun initial ()
  (format t "----- test start -----~%~%")
  (setf *test-count* 0
	*ok-count* 0
	*ng-count* 0
	*err-count* 0))

;;; result と collect が等しいかチェックする
(defun check (test expr collect)
  (when (null test) (setf test #'equal))
  (incf *test-count*)
  (format t "~a~%=> " expr)
  (handler-case
      (let ((result (eval expr)))
	(cond
	 ((funcall test result collect)
	  (incf *ok-count*)
	  (format t "~a OK~%~%" result))
	 (t
	  (incf *ng-count*)
	  (format t "~a NG, ~a~%~%" result collect))))
    (error (c)
	   (incf *err-count*)
	   (format t "ERROR: ~a~%~%" c))))

;;; テストの実行
(defmacro run (expr collect &key (test nil))
  `(mintst::check ,test  ',expr ,collect))

;;; 終了
(defun final ()
  (format t
	  "----- test end -----~%TEST: ~d~%OK: ~d~%NG: ~d~%ERR: ~d~%"
	  *test-count* *ok-count* *ng-count* *err-count*))

