;;;
;;; arc.lisp : 適応型 rangecoder
;;;
;;; Copyright (c) 2023 Makoto Hiroi
;;;
;;; Released under the MIT license
;;; https://opensource.org/license/mit/
;;;
(provide :arc)
(defpackage :arc (:use :cl :bitree :rangecoder))
(in-package :arc)
(export '(arc-encode
	  arc-decode))

;;; 更新
(defun arc-update (bt c limit inc)
  (bitree-add bt c inc)
  (when (<= limit (bitree-total bt))
    (bitree-half bt)))

;;; 適応型レンジコーダの符号化
(defun arc-encode (rc bt c &key (limit min-range) (inc 1))
  (let ((temp (floor (range-coder-range rc)
                     (bitree-total bt))))
    (incf (range-coder-low rc)
          (* (bitree-sum bt c) temp))
    (setf (range-coder-range rc)
          (* (bitree-get bt c) temp))
    (encode-normalize rc)
    (arc-update bt c limit inc)))

;;; 適応型レンジコーダの符号化
(defun arc-decode (rc bt &key (limit min-range) (inc 1))
  (let ((temp (floor (range-coder-range rc)
                     (bitree-total bt))))
    (multiple-value-bind
        (c sum)
        (bitree-find bt (floor (range-coder-low rc) temp))
      (decf (range-coder-low rc)
            (* sum temp))
      (setf (range-coder-range rc)
            (* (bitree-get bt c) temp))
      (decode-normalize rc)
      (arc-update bt c limit inc)
      c)))

