;;;
;;; brc_tst.lisp : binary range coder のテスト
;;;
;;; Copyright (C) 2023 Makoto Hiroi
;;;
;;; Released under the MIT license
;;; https://opensource.org/license/mit/
;;;
(provide :brc_tst)
(defpackage :brc_tst (:use :cl :mintst :brc :rangecoder))
(in-package :brc_tst)
(export '(test))

(defvar bc nil)
(defvar rc nil)
(defvar bm nil)
(defvar am nil)
(defvar bs nil)
(defvar gm nil)

(defun test ()
  (initial)
  (run (bit-context-p (setq bc (make-bit-context))) t)
  (call-with-byte-output-file
   "brc_test.en"
   (lambda (out)
     (call-with-range-encoder
      out
      (lambda (r)
	(setq rc r)
	(run (bit-encode rc bc 0) nil)
	(run (bit-encode rc bc 1) nil)
	(run (bit-encode rc bc 0) nil)
	(run (bit-encode rc bc 0) nil)
	(run (bit-encode rc bc 1) nil)
	(run (bit-encode rc bc 1) nil)
	(run (bit-encode rc bc 1) nil)
	(run (bit-encode rc bc 0) nil)))))
  (setq bc (make-bit-context))
  (call-with-byte-input-file
   "brc_test.en"
   (lambda (in)
     (call-with-range-decoder
      in
      (lambda (r)
	(setq rc r)
	(run (bit-decode rc bc) 0)
	(run (bit-decode rc bc) 1)
	(run (bit-decode rc bc) 0)
	(run (bit-decode rc bc) 0)
	(run (bit-decode rc bc) 1)
	(run (bit-decode rc bc) 1)
	(run (bit-decode rc bc) 1)
	(run (bit-decode rc bc) 0)))))
  ;; binary model のテスト
  (run (binary-model-p (setq bm (initialize-binary-model 8))) t)
  (call-with-byte-output-file
   "bm_test.en"
   (lambda (out)
     (call-with-range-encoder
      out
      (lambda (r)
	(setq rc r)
	(run (bm-encode rc bm 0) nil)
	(run (bm-encode rc bm 1) nil)
	(run (bm-encode rc bm 2) nil)
	(run (bm-encode rc bm 3) nil)
	(run (bm-encode rc bm 4) nil)
	(run (bm-encode rc bm 5) nil)
	(run (bm-encode rc bm 6) nil)
	(run (bm-encode rc bm 7) nil)))))
  (setq bm (initialize-binary-model 8))
  (call-with-byte-input-file
   "bm_test.en"
   (lambda (in)
     (call-with-range-decoder
      in
      (lambda (r)
	(setq rc r)
	(run (bm-decode rc bm) 0)
	(run (bm-decode rc bm) 1)
	(run (bm-decode rc bm) 2)
	(run (bm-decode rc bm) 3)
	(run (bm-decode rc bm) 4)
	(run (bm-decode rc bm) 5)
	(run (bm-decode rc bm) 6)
	(run (bm-decode rc bm) 7)))))
  ;; alpha model のテスト
  (run (alpha-model-p (setq am (initialize-alpha-model 8))) t)
  (call-with-byte-output-file
   "am_test.en"
   (lambda (out)
     (call-with-range-encoder
      out
      (lambda (r)
	(setq rc r)
	(run (alpha-encode rc am 0) nil)
	(run (alpha-encode rc am 1) nil)
	(run (alpha-encode rc am 2) nil)
	(run (alpha-encode rc am 3) nil)
	(run (alpha-encode rc am 4) nil)
	(run (alpha-encode rc am 5) nil)
	(run (alpha-encode rc am 6) nil)
	(run (alpha-encode rc am 7) nil)))))
  (setq am (initialize-alpha-model 8))
  (call-with-byte-input-file
   "am_test.en"
   (lambda (in)
     (call-with-range-decoder
      in
      (lambda (r)
	(setq rc r)
	(run (alpha-decode rc am) 0)
	(run (alpha-decode rc am) 1)
	(run (alpha-decode rc am) 2)
	(run (alpha-decode rc am) 3)
	(run (alpha-decode rc am) 4)
	(run (alpha-decode rc am) 5)
	(run (alpha-decode rc am) 6)
	(run (alpha-decode rc am) 7)))))
  ;; bits model のテスト
  (run (bits-model-p (setq bs (initialize-bits-model 3))) t)
  (call-with-byte-output-file
   "bs_test.en"
   (lambda (out)
     (call-with-range-encoder
      out
      (lambda (r)
	(setq rc r)
	(run (bits-encode rc bs 0) nil)
	(run (bits-encode rc bs 1) nil)
	(run (bits-encode rc bs 2) nil)
	(run (bits-encode rc bs 3) nil)
	(run (bits-encode rc bs 4) nil)
	(run (bits-encode rc bs 5) nil)
	(run (bits-encode rc bs 6) nil)
	(run (bits-encode rc bs 7) nil)))))
  (setq bs (initialize-bits-model 3))
  (call-with-byte-input-file
   "bs_test.en"
   (lambda (in)
     (call-with-range-decoder
      in
      (lambda (r)
	(setq rc r)
	(run (bits-decode rc bs) 0)
	(run (bits-decode rc bs) 1)
	(run (bits-decode rc bs) 2)
	(run (bits-decode rc bs) 3)
	(run (bits-decode rc bs) 4)
	(run (bits-decode rc bs) 5)
	(run (bits-decode rc bs) 6)
	(run (bits-decode rc bs) 7)))))
  ;; gamma model のテスト
  (run (gamma-model-p (setq gm (initialize-gamma-model 256))) t)
  (call-with-byte-output-file
   "gm_test.en"
   (lambda (out)
     (call-with-range-encoder
      out
      (lambda (r)
	(setq rc r)
	(run (gamma-encode rc gm 0) nil)
	(run (gamma-encode rc gm 15) nil)
	(run (gamma-encode rc gm 16) nil)
	(run (gamma-encode rc gm 31) nil)
	(run (gamma-encode rc gm 32) nil)
	(run (gamma-encode rc gm 63) nil)
	(run (gamma-encode rc gm 64) nil)
	(run (gamma-encode rc gm 255) nil)))))
  (setq gm (initialize-gamma-model 256))
  (call-with-byte-input-file
   "gm_test.en"
   (lambda (in)
     (call-with-range-decoder
      in
      (lambda (r)
	(setq rc r)
	(run (gamma-decode rc gm) 0)
	(run (gamma-decode rc gm) 15)
	(run (gamma-decode rc gm) 16)
	(run (gamma-decode rc gm) 31)
	(run (gamma-decode rc gm) 32)
	(run (gamma-decode rc gm) 63)
	(run (gamma-decode rc gm) 64)
	(run (gamma-decode rc gm) 255)))))
  (final))

