;;;
;;; treemap_tst.lisp : treemap のテスト
;;;
;;; Copyright (C) 2023 Makoto Hiroi
;;;
;;; Released under the MIT license
;;; https://opensource.org/license/mit/
;;;
(provide :treemap_tst)
(defpackage :treemap_tst (:use :cl :mintst :treemap))
(in-package :treemap_tst)
(export '(test))

(defvar ts nil)
(defvar ks nil) 
(defvar vs nil)

(defun test-sub ()
  (run (tree-emptyp ts) t)
  (run (tree-count ts) 0)
  (run (progn
	 (setq ks '("foo" "bar" "baz" "oops")
	       vs '(10 20 30 40))
	 (mapc (lambda (k v) (tree-set ts k v)) ks vs)
	 (tree-count ts))
       4)
  (run (tree-emptyp ts) nil)
  (run (mapcar (lambda (k) (tree-exists ts k)) '("foo" "Foo")) '(t nil))
  (run (mapcar (lambda (k) (tree-exists ts k)) ks) '(t t t t))
  (run (mapcar (lambda (k) (tree-get ts k)) '("foo" "Foo")) '(10 nil))
  (run (mapcar (lambda (k) (tree-get ts k)) ks) '(10 20 30 40))
  (run (tree-min ts) '("bar" . 20))
  (run (tree-max ts) '("oops" . 40))
  (run (let ((a nil))
	 (tree-mapc ts (lambda (k v) (push (cons k v) a)))
	 (reverse a))
       '(("bar" . 20) ("baz" . 30) ("foo" . 10) ("oops" . 40)))
  (run (tree-fold-right ts nil (lambda (a k v) (cons (cons k v) a)))
       '(("bar" . 20) ("baz" . 30) ("foo" . 10) ("oops" . 40)))
  (run (tree-fold-left ts nil (lambda (a k v) (cons (cons k v) a)))
       (reverse '(("bar" . 20) ("baz" . 30) ("foo" . 10) ("oops" . 40))))
  (run (tree-max-del ts) t)
  (run (tree-count ts) 3)
  (run (tree-max ts) '("foo" . 10))
  (run (tree-min-del ts) t)
  (run (tree-count ts) 2)
  (run (tree-min ts) '("baz" . 30))
  (run (tree-del ts "foo") t)
  (run (tree-count ts) 1)
  (run (tree-min ts) '("baz" . 30))
  (run (tree-max ts) '("baz" . 30))
  (run (tree-del ts "baz") t)
  (run (tree-count ts) 0)
  (run (tree-emptyp ts) t)
  (run (tree-max ts) nil)
  (run (tree-min ts) nil)
  (run (progn
	 (mapc (lambda (k v) (tree-push ts k v)) ks vs)
	 (tree-count ts))
       4)
  (run (tree-push ts "Foo" 0) t)
  (run (tree-count ts) 5)
  (run (tree-get ts "Foo") '(0))
  (run (tree-push ts "Foo" 1) nil)
  (run (tree-count ts) 5)
  (run (tree-get ts "Foo") '(1 0))
  (run (tree-pop ts "Foo") 1)
  (run (tree-pop ts "Foo") 0)
  (run (tree-pop ts "Foo") nil)
  (run (tree-count ts) 5)
  (run (tree-del ts "Foo") t)
  (run (tree-count ts) 4)
  (run (tree-clear ts) 0)
  (run (tree-count ts) 0)
  (run (tree-emptyp ts) t))

(defun test ()
  (initial)
  (run (treemap-p (setq ts (make-treemap #'string= #'string<))) t)
  (test-sub)
  (run (splaytree-p (setq ts (make-splaytree #'string= #'string<))) t)
  (test-sub)
  (final))

  
