;;;
;;; trie_tst.lisp : trie のテスト
;;;
;;; Copyright (C) 2023 Makoto Hiroi
;;;
;;; Released under the MIT license
;;; https://opensource.org/license/mit/
;;;
(provide :trie_tst)
(defpackage :trie_tst (:use :cl :mintst :trie))
(in-package :trie_tst)
(export '(test))

(defvar ts nil)
(defvar ks nil) 
(defvar vs nil)
(defvar cs nil)

(defun to-str (xs)
  (if (stringp (car xs))
      (apply #'concatenate 'string xs)
    (coerce xs 'string)))

(defun test-sub ()
  (run (trie-emptyp ts) t)
  (run (trie-count ts) 0)
  (run (progn
	 (setq ks '("foo" "bar" "baz" "oops")
	       vs '(10 20 30 40))
	 (mapc (lambda (k v) (trie-set ts k v)) ks vs)
	 (trie-count ts))
       4)
  (run (trie-emptyp ts) nil)
  (run (mapcar (lambda (k) (trie-exists ts k)) '("foo" "Foo")) '(t nil))
  (run (mapcar (lambda (k) (trie-exists ts k)) ks) '(t t t t))
  (run (mapcar (lambda (k) (trie-get ts k)) '("foo" "Foo")) '(10 nil))
  (run (mapcar (lambda (k) (trie-get ts k)) ks) '(10 20 30 40))
  (cond
   ((ternary-p ts)
    (run (let ((a nil))
	   (trie-mapc ts (lambda (k v) (push (cons (to-str k) v) a)))
	   (reverse a))
	 '(("bar" . 20) ("baz" . 30) ("foo" . 10) ("oops" . 40)))
    (run (trie-fold ts nil (lambda (a k v) (cons (cons (to-str k) v) a)))
	 '(("oops" . 40) ("foo" . 10) ("baz" . 30) ("bar" . 20))))
   (t
    (run (let ((a nil))
	   (trie-mapc ts (lambda (k v) (push (cons (to-str k) v) a)))
	   (reverse a))
	 '(("oops" . 40) ("baz" . 30) ("bar" . 20) ("foo" . 10)))
    (run (trie-fold ts nil (lambda (a k v) (cons (cons (to-str k) v) a)))
	 '(("foo" . 10) ("bar" . 20) ("baz" . 30) ("oops" . 40)))))
  (run (trie-del ts "foo") t)
  (run (trie-exists ts "foo") nil)
  (run (trie-count ts) 3)
  (run (trie-del ts "bar") t)
  (run (trie-exists ts "bar") nil)
  (run (trie-count ts) 2)
  (run (trie-del ts "baz") t)
  (run (trie-exists ts "baz") nil)
  (run (trie-count ts) 1)
  (run (trie-del ts "oops") t)
  (run (trie-exists ts "oops") nil)
  (run (trie-count ts) 0)
  (run (trie-emptyp ts) t)

  (run (progn
	 (mapc (lambda (k v) (trie-push ts k v)) ks vs)
	 (trie-count ts))
       4)
  (run (trie-push ts "Foo" 0) t)
  (run (trie-count ts) 5)
  (run (trie-get ts "Foo") '(0))
  (run (trie-push ts "Foo" 1) nil)
  (run (trie-count ts) 5)
  (run (trie-get ts "Foo") '(1 0))
  (run (trie-pop ts "Foo") 1)
  (run (trie-pop ts "Foo") 0)
  (run (trie-pop ts "Foo") nil)
  (run (trie-count ts) 5)
  (run (trie-del ts "Foo") t)
  (run (trie-count ts) 4)
  (run (trie-clear ts) 0)
  (run (trie-count ts) 0)
  (run (trie-emptyp ts) t)
  ;;
  (run (progn
	 (mapc (lambda (k) (trie-set ts k t)) '("abcde" "abcd" "abc" "ab" "a"))
	 (trie-count ts))
       5)
  (run (let ((a nil))
	 (trie-common-prefix ts "abc" (lambda (k v) (push (cons (to-str k) v) a)))
	 (reverse a))
       '(("abc" . t) ("abcd" . t) ("abcde" . t)))
  (run (trie-count (setq cs (trie-copy ts))) 5)
  (run (mapcar (lambda (k) (trie-get cs k)) '("abcde" "abcd" "abc" "ab" "a"))
       '(t t t t t))
  (run (trie-del cs "abcde") t)
  (run (trie-exists cs "abcde") nil)
  (run (trie-exists ts "abcde") t))

(defun test ()
  (initial)
  (run (trie-p (setq ts (make-trie))) t)
  (test-sub)
  (run (patricia-p (setq ts (make-patricia))) t)
  (test-sub)
  (run (ternary-p (setq ts (make-ternary))) t)
  (test-sub)
  (final))

  
