;;;
;;; utils_tst.lisp : utils のテスト
;;;
;;; Copyright (C) 2023 Makoto Hiroi
;;;
;;; Released under the MIT license
;;; https://opensource.org/license/mit/
;;;
(provide :utils_tst)
(defpackage :utils_tst (:use :cl :mintst :utils))
(in-package :utils_tst)
(export '(test))

(defun square (x) (* x x))

;;; 実行
(defun test ()
  (initial)
  ;; リストの生成 
  (run (xcons 1 2) '(2 . 1))
  (run (iota 3) '(0 1 2))
  (run (iota 4 :start 1) '(1 2 3 4))
  (run (iota 5 :start 2 :step 2) '(2 4 6 8 10))
  (run (tabulate #'identity 3) '(0 1 2))
  (run (tabulate #'identity 4 :start 1) '(1 2 3 4))
  (run (tabulate #'square 5 :step 2) '(0 4 16 36 64))
  (run (unfold (lambda (x) (>= x 4)) #'identity #'1+ 0) '(0 1 2 3))
  (run (unfold (lambda (x) (>= x 5)) #'square #'1+ 1) '(1 4 9 16))
  ;; リストの操作
  (run (take '(1 2 3 4 5 6) 0) nil)
  (run (take '(1 2 3 4 5 6) 3) '(1 2 3))
  (run (take '(1 2 3 4 5 6) 6) '(1 2 3 4 5 6))
  (run (drop '(1 2 3 4 5 6) 0) '(1 2 3 4 5 6))
  (run (drop '(1 2 3 4 5 6) 3) '(4 5 6))
  (run (drop '(1 2 3 4 5 6) 6) nil)
  (run (take-while #'evenp '(1 3 5 7 9)) nil)
  (run (take-while #'oddp '(1 3 5 7 9)) '(1 3 5 7 9))
  (run (take-while #'evenp '(2 4 6 8 1 3 5 7)) '(2 4 6 8))
  (run (drop-while #'evenp '(1 3 5 7 9)) '(1 3 5 7 9))
  (run (drop-while #'oddp '(1 3 5 7 9)) nil)
  (run (drop-while #'evenp '(2 4 6 8 1 3 5 7)) '(1 3 5 7))
  (run (multiple-value-list (split-at '(1 2 3 4 5 6) 0))
       '(nil (1 2 3 4 5 6)))
  (run (multiple-value-list (split-at '(1 2 3 4 5 6) 3))
       '((1 2 3) (4 5 6)))
  (run (multiple-value-list (split-at '(1 2 3 4 5 6) 6))
       '((1 2 3 4 5 6) nil))
  (run (multiple-value-list (split-if #'evenp '(2 4 6 8 1 3 5 7)))
       '((2 4 6 8) (1 3 5 7)))
  (run (multiple-value-list (split-if #'oddp '(2 4 6 8 1 3 5 7)))
       '(nil (2 4 6 8 1 3 5 7)))
  (run (multiple-value-list (split-if #'oddp '(1 3 5 7 9)))
       '((1 3 5 7 9) nil))
  (run (multiple-value-list (split-if-not #'oddp '(2 4 6 8 1 3 5 7)))
       '((2 4 6 8) (1 3 5 7)))
  (run (multiple-value-list (split-if-not #'evenp '(2 4 6 8 1 3 5 7)))
       '(nil (2 4 6 8 1 3 5 7)))
  (run (multiple-value-list (split-if-not #'evenp '(1 3 5 7 9)))
       '((1 3 5 7 9) nil))
  (run (multiple-value-list (partition #'evenp '(0 1 2 3 4 5 6 7)))
       '((0 2 4 6) (1 3 5 7)))
  (run (multiple-value-list (partition #'evenp '(0 2 4 6 8)))
       '((0 2 4 6 8) nil))
  (run (multiple-value-list (partition #'oddp '(0 2 4 6 8)))
       '(nil (0 2 4 6 8)))
  (run (insert '(1 3 5 7) 0 #'<) '(0 1 3 5 7))
  (run (insert '(1 3 5 7) 4 #'<) '(1 3 4 5 7))
  (run (insert '(1 3 5 7) 8 #'<) '(1 3 5 7 8))
  (run (insert-at '(1 3 5 7) 0 9) '(9 1 3 5 7))
  (run (insert-at '(1 3 5 7) 2 9) '(1 3 9 5 7))
  (run (insert-at '(1 3 5 7) 4 9) '(1 3 5 7 9))
  (run (remove-at '(1 3 5 7 9) 0) '(3 5 7 9))
  (run (remove-at '(1 3 5 7 9) 2) '(1 3 7 9))
  (run (remove-at '(1 3 5 7 9) 4) '(1 3 5 7))
  (run (selects '(a)) '((a)))
  (run (selects '(a b)) '((a b) (b a)))
  (run (selects '(a b c)) '((a b c) (b a c) (c a b)))
  (run (power-set '(a b c d))
       '(NIL (D) (C) (C D) (B) (B D) (B C) (B C D) (A) (A D) (A C)
	     (A C D) (A B) (A B D) (A B C) (A B C D)))
  (run (fold-left #'+ 0 '(1 2 3 4 5)) 15)
  (run (fold-left #'xcons nil '(a b c)) '(c b a))
  (run (fold-left (lambda (a x y) (cons (cons x y) a)) nil '(a b c) '(1 2 3))
       '((C . 3) (B . 2) (A . 1)))
  (run (fold-right #'+ 0 '(1 2 3 4 5)) 15)
  (run (fold-right (lambda (a x) (cons x a)) nil '(a b c))
       '(a b c))
  (run (fold-right (lambda (a x y) (cons (cons x y) a)) nil '(a b c) '(1 2 3))
       '((A . 1) (B . 2) (C . 3)))
  (run (scan-left #'+ 0 (iota 10 :start 1))
       '(0 1 3 6 10 15 21 28 36 45 55))
  (run (scan-left (lambda (a x y) (cons (cons x y) a)) nil '(a b c) '(1 2 3))
       '(NIL ((A . 1)) ((B . 2) (A . 1)) ((C . 3) (B . 2) (A . 1))))
  (run (scan-right #'+ 0 (iota 10 :start 1))
       '(55 54 52 49 45 40 34 27 19 10 0))
  (run (scan-right (lambda (a x y) (cons (cons x y) a)) nil '(a b c) '(1 2 3))
       '(((A . 1) (B . 2) (C . 3)) ((B . 2) (C . 3)) ((C . 3)) NIL))

  ;; ベクタの操作
  (run (vector-iota 3) #(0 1 2) :test #'equalp)
  (run (vector-iota 4 :start 1) #(1 2 3 4) :test #'equalp)
  (run (vector-iota 5 :start 2 :step 2) #(2 4 6 8 10) :test #'equalp)
  (run (vector-tabulate #'identity 3) #(0 1 2) :test #'equalp)
  (run (vector-tabulate #'identity 4 :start 1) #(1 2 3 4) :test #'equalp)
  (run (vector-tabulate #'square 5 :step 2) #(0 4 16 36 64) :test #'equalp)
  (run (vector-unfold 4 #'identity #'1+ 0) #(0 1 2 3) :test #'equalp)
  (run (vector-unfold 5 #'square #'1+ 1) #(1 4 9 16 25) :test #'equalp)
  (run (vector-fold-left #'+ 0 #(1 2 3 4 5)) 15)
  (run (vector-fold-left #'xcons nil #(a b c)) '(c b a))
  (run (vector-fold-left (lambda (a x y) (cons (cons x y) a)) nil #(a b c) #(1 2 3))
       '((C . 3) (B . 2) (A . 1)))
  (run (vector-fold-right #'+ 0 #(1 2 3 4 5)) 15)
  (run (vector-fold-right (lambda (a x) (cons x a)) nil #(a b c))
       '(a b c))
  (run (vector-fold-right (lambda (a x y) (cons (cons x y) a)) nil #(a b c) #(1 2 3))
       '((A . 1) (B . 2) (C . 3)))
  (run (vector-map-with-index (lambda (i x) (list i x)) #(a b c))
       #((0 A) (1 B) (2 C)) :test #'equalp)
  (run (vector-map-with-index (lambda (i x y) (list i x y)) #(a b c d) #(10 20 30))
       #((0 A 10) (1 B 20) (2 C 30)) :test #'equalp)
  (run (let ((a nil))
	 (vector-mapc-with-index (lambda (i x) (push (list i x) a)) #(a b c))
	 (reverse a))
       '((0 A) (1 B) (2 C)))
  (run (let ((a nil))
	 (vector-mapc-with-index (lambda (i x y) (push (list i x y) a)) #(a b c d) #(10 20 30))
	 (reverse a))
       '((0 A 10) (1 B 20) (2 C 30)))
  (run (vector-swap #(1 2 3 4 5) 0 4) #(5 2 3 4 1) :test #'equalp)
  (run (vector-swap #(1 2 3 4 5) 1 3) #(1 4 3 2 5) :test #'equalp)
  (run (vector-swap #(1 2 3 4 5) 2 2) #(1 2 3 4 5) :test #'equalp)
  (run (multiple-value-list (vector-partition #'evenp #(1 2 3 4 5 6)))
       '(3 #(6 2 4 3 5 1)) :test #'equalp)
  (run (multiple-value-list (vector-partition #'evenp #(0 2 4 6 8 10)))
       '(6 #(0 2 4 6 8 10)) :test #'equalp)
  (run (multiple-value-list (vector-partition #'evenp #(1 3 5 7 9 11)))
       '(0 #(1 3 5 7 9 11)) :test #'equalp)
  (run (vector-bsearch 5 #(1 2 3 4 5 6 7 8 9)) 4)
  (run (vector-bsearch 1 #(1 2 3 4 5 6 7 8 9)) 0)
  (run (vector-bsearch 9 #(1 2 3 4 5 6 7 8 9)) 8)
  (run (vector-bsearch 1 #(1 2 3 4 5 6 7 8 9) :start 1) nil)
  (run (vector-bsearch 9 #(1 2 3 4 5 6 7 8 9) :end 8) nil)
  (run (vector-bsearch 5.5 #(1 2 3 4 5 6 7 8 9)) nil)
  (run (dotimes (x 12)
	 (when (equalp (vector-shuffle (vector 1 2 3)) #(3 2 1)) (return t)))
       t)
  (run (dotimes (x 12)
	 (when (equalp (vector-shuffle (vector 1 2 3)) #(1 2 3)) (return t)))
       t)
  (final))
					
